; This is the basic outline for a program that can determine the 
; frequency of an input, via input capture. The input capture has
; been selected to capture on the 16th rising edge. This is useful
; for high frequency inputs, where an interrupt on each rising edge
; would not be able to be serviced (at that rate). This particular
; example can support an input signal with a period of approximatly
; 625 nS. Without the divide by 16 selected, this is approximatly
; 10 us. This period time increases (frequency decreases) as the
; overhead in the main routine increases. 
;
; This routine uses an 8-bit register to count the times that timer3
; overflowed. At the Max crystal frequency of 16 MHz, this gives an
; overflow time of (16)(2**8 + 1)(2**16)(250 nS) > 67.37 sec. If
; measurement of longer time intervals is required, the overflow
; counter could be extended to 16 (or more) bits.
;
; Timer 3 in this example is a free running timer. The input 
; capture is generated on the RB1/CAP2 pin. There is a flag
; that specifies if this is the 1st or 2nd capture.
; The first capture is the start of the period measurement. The
; second capture value gives the end of the period. In this type
; of measurement If the 2nd capture value < the 1st captue value
; then the overflow counter should be decremented.
;
; Do the EQUate table
;
IC2OF		EQU	0x20		; T3 overflow register
IC2BH		EQU	0x21		; T3 ICA2 MSB register (2nd Cap)
IC2BL		EQU	0x22		; T3 ICA2 LSB register
IC2AH		EQU	0x23		; T3 ICB2 MSB register (1st Cap)
IC2AL		EQU	0x24		; T3 ICB2 LSB register
T3OFLCNTR	EQU 0x25		; Temperay T3 overflow register
;
FLAG_REG	EQU 0x26		; Register that has the Flag bits
;
;    FLAG_REG  bit   7   6   5   4   3   2   1   0
;                    -   -   -   -   -   -  UFL CAP1
;    CAP1 = 0,  1st Capture
;         = 1,  2nd Capture
;
;    UFL = 0,  No Underflow
;        = 1,  Underflow during subtract
;
END_OF_PROG_MEM		EQU 0x07FF
;
;
ALUSTA		EQU	0x04
CPUSTA		EQU	0x06
INTSTA		EQU	0x07
W			EQU	0x0A
;
PORTB		EQU	0x12		; Bank 0
;
PIR			EQU	0x16		; Bank 1
PIE			EQU	0x17
;
TMR3L		EQU	0x12		; Bank 2
TMR3H		EQU	0x13
T3PRL		EQU 0x16
T3PRH		EQU 0x17
;
CA2L		EQU	0x14		; Bank 3
CA2H		EQU	0x15
TCON1		EQU	0x16
TCON2		EQU	0x17
 PAGE
		ORG		0x0000			; Origin for the RESET vector
		GOTO	START			; On reset, go to the start of
								;   the program	
		ORG		0x0008			; Origin for the external RA0/INT
								;   interrupt vector
		GOTO	EXT_INT			; Goto the ext. interrupt
								;   on RA0/INT routine
		ORG		0x0010			; Origin for the RTCC
								;   overflow interrupt vector
		GOTO	RTCCINT			; Goto the RTCC overflow interrupt
								;   routine
		ORG		0x0018			; Origin for the external
								;   RA1/RT interrupt vector
		GOTO	RT_INT			; Goto the ext. interrupt on 
								;   RA1/RT routine
		ORG		0x0020			; Origin for the interrupt vector
								;   of any enabled peripheral
		GOTO	PER_INT			; Goto the interrupt from a
								;   peripheral routine
 PAGE
		ORG		0x0028			; Origin for the top of
								;   program memory
START	BSF		CPUSTA,4		; Disable ALL interrupts via the
								;   Global Interrupt Disable 
								;   (GLINTD) bit.
								;
MAIN							; Place Main program here
		MOVLB	3				; Select register Bank 3
		CLRF	TCON2,0			; Stop the timers, Single Capture
		MOVLW	0x0F0			; Initalize TCON1 so that
		MOVWF	TCON1			;   T1 (8-bit), T2 (8-bit), 
								;   and T3 run off the internal
								;   system clock. Capture2 
								;   captures on the 16th rising edge.
;
; Initalize Timer 3, load the timer with the number of cycles that 
; the device executes (from RESET) before the timer is turned on
; Therefore the offset is required due to software overhead.
;
		MOVLB	2				; Select register Bank 2
		CLRF	W,0				; Clear the W register
		MOVWF	FLAG_REG		; Initalize to 0
		MOVWF	TMR3H			; Timer3 MSB = 0
		MOVLW	0x00			; Timer3 LSB = Offset
		MOVWF	TMR3L			;
;
; Load the Timer 3 period register with 0xFFFF, which will give an
; interrupt on the overflow of Timer3
;
		MOVLW	0xFF			;
		MOVWF	T3PRH			;
		MOVWF	T3PRL			;
;
; the timer should be started and interrupts enabled.
;
		MOVLB	3				; Select register Bank 3
		BSF		TCON2,2			; Turn on timer 3.
		BSF		INTSTA,3		; Turn on Peripheral Interrupts
		MOVLB	1				; Select register Bank 1
		MOVLW	0x48			; Enable Caputure 2 and Timer3
		MOVWF	PIE				;    Interrupts (when GLINTD = 0)
;
; This is where you would do the things you wanted to do. 
; this example will only loop waiting for the interrupts.
;
WAIT	BCF		CPUSTA,4		; Enable ALL interrupts
		GOTO	WAIT			; Loop here waiting for a timer
								;   Interrupt
 PAGE
;
; The interrupt routine for any peripheral interrupt, This routine
; only deals with Timer3 (T3) interrupts.
;
; Time required to execute interrupt routine. Not including
; interrupt latency (time to enter into the interrupt routine)
;
; 	case1 - only T3 overflow             = 12 cycles
;	case2 - 1st capture                  = 14 cycles
;	case3 - 2nd capture                  = 30 cycles
;	case4 - T3 overflow and 1st capture  = 34 cycles
;	case5 - T3 overflow and 2nd capture  = 50 cycles
;
;
PER_INT	MOVLB	1				; Select register Bank 1
		BTFSC	PIR,6			; Did T3 overflow?
								;   If not skip next Instruction
		GOTO	T3OVFL			; Inc overflow cntr and clear flag
CK_CAP	BTFSS	PIR,3			; Did the RB1/CAP2 pin cause an
								;   interrupt?
		RETFIE					; No RB1/CAP2 interrupt,
								;   Return from Interrupt
;
; This potion of the code takes the 1st capture and stores its
; value in register pair IC2AH:IC2AL. When the 2nd capture
; is take, its value is stored in register pair IC2BH:IC2BL.
; A 16-bit subtract is performed, with the final 24-bit result
; being stored in IC2OF:IC2BH:IC2BL. This value will no longer
; be correct after the next capture occurs (IC2BH:IC2BL will
; change), so the main routine must utilize this value before
; it changes.
;
CAPTURE	BCF		PIR,3			; Clear Capture2 interrupt flag
		MOVLB	3				; Select register Bank 3
		BTFSC	FLAG_REG,0		; 1st or 2nd capture2?
		GOTO	CAP2			; It was the 2nd Capture
CAP1	MOVPF	CA2L,IC2AL		; Move the captured value to
		MOVPF	CA2H,IC2AH		;   temporary registers
		BSF		FLAG_REG,0		; Have 1st capture2
		RETFIE					; Return from Interrupt
								;
 PAGE
CAP2	MOVPF	CA2L,IC2BL		; Move the captured value to
		MOVPF	CA2H,IC2BH		;   temporary registers
								;   (to pervent being overwritten)
								;
		CALL	SUB16			; Call routine which subtracts
								;   2 16-bit numbers.
		BTFSC	FLAG_REG,1		; Underflow during SUB16?
		DECF	T3OFLCNTR,1		; Since underflow, decrement the
								;   overflow counter.
		CLRF	FLAG_REG,1		; Clear the flag bits for
								;   underflow and 2nd capture2
		MOVFP	T3OFLCNTR,W		; Store the T3 input capture
		MOVPF	W,IC2OF			;   overflow value in IC2OF
		CLRF	T3OFLCNTR,0		; Clear the Data register which
								;   counts how many times Timer 3
								;   overflows.
		RETFIE					; Return from interrupt
;
; When Timer 3 has overflowed, the overflow counter only should
; be incremented when the overflow occurs after a capture 1 
; but before the capture 2. The 4 possible cases when entering
; the T3OVFL section of the PER_INT routine are as follows:
;   Case 1: T3 overflow (only) and FLAG_REG.0 = 0 (waiting
;           for Capture 1 to occur). Do Not increment counter
;   Case 2: T3 overflow (only) and FLAG_REG.0 = 1 (waiting
;           for Capture 2 to occur). Increment counter
;   Case 3: T3 Overflow happened after Capture. Do Not
;           increment overflow counter
;   Case 4: T3 Overflow occured before Capture 2 and FLAG_REG.0 = 1
;           (waiting for Capture 2 to occur). Increment counter

;
T3OVFL	BCF		PIR,6			; Clear Overflow interrupt flag
		BTFSS	PIR,3			; Did the RB1/CAP2 pin also
								;   cause an interrupt?
		GOTO	FR0				; No, Check if between 1st
								;   and 2nd capture
		MOVLB	3				; Bank 3
		CLRF	W,0				; W = 0
		CPFSEQ	CA2H			; if CA2H = 0, overflow happened
		GOTO	FR0				;   first, must check FLAG_REG
								;   bit 0
		MOVLB	1				; Back to bank 1
		GOTO	CAPTURE			; Capture happened first, do NOT
								;   Increment overflow counter
								;   and do capture routine		
FR0		BTFSC	FLAG_REG,0		; Between Capture 1 and Capture 2?
		INCF	T3OFLCNTR,1		; Yes, Inc. the overflow counter
		RETFIE					; Return from overflow interrupt
;
SUB16	MOVFP	IC2AL,W			; Do the 16-bit subtraction
		SUBWF	IC2BL,1			; 
		MOVFP	IC2AH,W			;
		SUBWFB	IC2BH,1			;
		BTFSS	ALUSTA,0		; Is the result pos. or neg. ? 
		BSF		FLAG_REG,1		; neg., Set the underflow flag
		RETURN					; Return from the subroutine
 PAGE
;
; Other Interrupt routines. (Not utilized in this example)
;
EXT_INT	RETFIE					; RA0/INT interrupt routine
								;   (NOT used in this program)
RTCCINT	RETFIE					; RTCC overflow interrupt routine
								;   (NOT used in this program)
RT_INT	RETFIE					; RA1/RT interrupt routine
								;   (NOT used in this program)
								;
SRESET	GOTO	START			; If program became lost, goto
								;   START and reinitalize.
;
;
; When the executed address is NOT in the program range, the
; 16-bit address should contain all 1's (a CALL 0x1FFF). At
; this location you could branch to a routine to recover or
; shut down from the invalid program execution.
;
		ORG		END_OF_PROG_MEM	; 
		GOTO	SRESET			; The program has lost it's mind,
								;   do a system reset
		END
