; This is the basic outline for a program that can determine the 
; frequency of an input, via input capture. This routine uses an
; 8-bit register to count the times that timer3 overflowed. At the
; Max crystal frequency of 16 MHz, this gives an overflow time of
; (2**16)(256 + 1)(250 nS) > 4.21 sec or a frequncy < 0.25 Hz. If
; measurement of longer time intervals is required, the overflow
; counter could be extended to 16 (or more) bits.
;
; Do the EQUate table
;
IC2OF		EQU	0x20		; T3 overflow data register
IC2H		EQU	0x21		; T3 IC2 MSB data register
IC2L		EQU	0x22		; T3 IC2 LSB data register
T3OFLCNTR	EQU 0x23		; Temperay T3 overflow register
;
END_OF_PROG_MEM		EQU		0x07FF
;

CPUSTA		EQU	0x06
INTSTA		EQU	0x07
W			EQU	0x0A
							;
PORTB		EQU	0x12		; Bank 0
							;
PIR			EQU	0x16		; Bank 1
PIE			EQU	0x17
							;
TMR3L		EQU	0x12		; Bank 2
TMR3H		EQU	0x13
							;
CA2L		EQU	0x14		; Bank 3
CA2H		EQU	0x15
TCON1		EQU	0x16
TCON2		EQU	0x17
;
		ORG		0x0000			; Origin for the RESET vector
		GOTO	START			; On reset, go to the start of
								;   the program	
		ORG		0x0008			; Origin for the external RA0/INT
								;   interrupt vector
		GOTO	EXT_INT			; Goto the ext. interrupt
								;   on RA0/INT routine
		ORG		0x0010			; Origin for the RTCC
								;   overflow interrupt vector
		GOTO	RTCCINT			; Goto the RTCC overflow interrupt
								;   routine
		ORG		0x0018			; Origin for the external
								;   RA1/RT interrupt vector
		GOTO	RT_INT			; Goto the ext. interrupt on 
								;   RA1/RT routine
		ORG		0x0020			; Origin for the interrupt vector
								;   of any enabled peripheral
		GOTO	PER_INT			; Goto the interrupt from a
								;   peripheral routine
		ORG		0x0028			; Origin for the top of
								;   program memory
START	BSF		CPUSTA,4		; Disable ALL interrupts via the
								;   Global Interrupt Disable 
								;   (GLINTD) bit.
								;
MAIN							; Place Main program here
		MOVLB	3				; Select register Bank 3
		CLRF	TCON2,0			; Stop the timers,
		BSF		TCON2,3			;   Dual input capture mode
		MOVLW	0x070			; Initalize TCON1 so that
		MOVWF	TCON1			;   T1 (8-bit), T2 (8-bit), 
								;   and T3 run off the internal
								;   system clock. Capture2 
								;   captures on the rising edge.
;
; Initalize Timer 3, so we can count clock cycles. For the 1st
; measurement Timer3 is not started exactly at the RA1/CAP2 edge
; transition. Therefore an offset is required due to software
; overhead (3 cycle).
;
		MOVLB	2				; Select register Bank 2
		CLRF	W,0				; Clear the W register
		MOVWF	TMR3H			; Timer3 MSB = 0
		MOVLW	0x03			; Timer3 LSB = Offset
		MOVWF	TMR3L			;
;
; We need to wait for an event to happen to start timer 3. This will
; be when the RA1/CAP2 pin to does a Low to High transition. This
; signifies the start of the first interval to measure. This is done
; by looping while the RA1/CAP2 pin is high, and then looping while
; it is low. When that 2nd loop is exited, there has been a low to
; high transition and the timer should be started and interrupts
; enabled.
;
		MOVLB	0				; Select register Bank 0
RA1HIGH	BTFSC	PORTB,1			; Is the CAP2 pin High?
		GOTO	RA1HIGH			; Loop here while CAP2 is High
RA1LOW	BTFSS	PORTB,1			; Is the CAP2 pin Low?
		GOTO	RA1LOW			; Loop here while CAP2 is Low
		MOVLB	3				; Select register Bank 3
		BSF		TCON2,2			; Turn on timer 3.
		BSF		INTSTA,3		; Turn on Peripheral Interrupts
		MOVLB	1				; Select register Bank 1
		MOVLW	0x48			; Enable Caputure 2 and Timer3
		MOVWF	PIE				;    Interrupts (when GLINTD = 0)
WAIT	BCF		CPUSTA,4		; Enable ALL interrupts
		GOTO	WAIT			; Loop here waiting for a timer
								;   Interrupt
;
; The interrupt routine for any peripheral interrupt, This routine
; only deals with Timer3 (T3) interrupts.
;
PER_INT	MOVLB	1				; Select register Bank 1
		BTFSC	PIR,6			; Did T3 overflow?
								;   If not skip next Instruction
		GOTO	T3OVFL			; Inc overflow cntr and clear flag
CK_CAP	BTFSS	PIR,3			; Did the RB1/CAP2 pin cause an
								;   interrupt?
		RETFIE					; No RB1/CAP2 interrupt,
								;   Return from Interrupt
;
; If Both a timer 3 overflow and CAP2 interrupt occured near the
; same time. Need to determine which one occured first.There are
; two possibilities when the PIR register has both the timer 3
; overflow interrupt and input capture 2 interrupt bits set.
;      1. the input capture occured and then timer 3 overflowed
;      2. timer 3 overflowed and then the input capture occured
;  in case 2. the MSB of the capture register would be 0h.
;
CAPTURE	MOVLB	3				; Select register Bank 3
		MOVPF	CA2L,IC2L		; Move the captured value to
		MOVPF	CA2H,IC2H		;   temporary registers
								;   (to pervent being overwritten)
		MOVLB	2				; Select register Bank 2
		MOVLW	0x000A			; Reload the Timer3 register with
		MOVWF	TMR3L			;   10, sinceit takes 10 cycles
								;   to reach this point from a
								;   timer3 interrupt
		CLRF	W,0				; Clear the W register
		MOVWF	TMR3H			; Timer3 MSB = 0
								;
		MOVLB	1				; Select register Bank 1
		CPFSEQ	IC2H			; If the high byte of the Capture
								;   register = 0,
								;   (ie Capture after overflow).
		GOTO	SDEC			; Skip the decrement instruction
		DECF	T3OFLCNTR,1		; Capture occured before the
								;   overflow, so decrement the
								;   overflow counter.
SDEC	MOVFP	T3OFLCNTR,W		; Store the T3 input capture
		MOVPF	W,IC2OF			;   overflow value in IC2OF
		CLRF	T3OFLCNTR,0		; Clear the Data register which
								;   counts how many times Timer 3
								;   overflows.
;
; Clear the Capture2 and T3 Overflow interrupt flags. T3 Overflow
; should be cleared again since the overflow may have occured
; while we were in this (PER_INT) interrupt routine. This would
; be between the compare of the T3 overflow interrupt flag and
; the resetting of the timer 3 counter, which is about 6 cycles.
;
		BCF		PIR,3			; Clear Capture2 interrupt flag
		BCF		PIR,6			; Clear Timer3 overflow interrupt
								;   flag
		RETFIE					; Return from interrupt
								;
;
; When Timer 3 has overflowed, the overflow counter should be
; incremented. Then the Timer 3 overflow flag, the source of 
; the interrupt, should be cleared. then return to see if a
; capture also occured.
;
; NOTE: Timer 3 has a process dependent silicon problem which
;       may give 2 overflow interrupts for each "real" overflow
;       interrupt. The first ("false") interrupt will occur at
;       the timer 0xFEFF to 0xFF00 transition. The second
;       interrupt then occursat the "real" overflow. A software
;       fix is included in this routine (T3OVFL). These additional
;       lines are designated by ;** as a comment. Once this has
;       been corrected, these lines of code can be removed.
;       The routine lable must remain.
;
T3OVFL	BCF		PIR,6			; Clear interrupt flag
		MOVLB	02				;** Switch to Bank 2
		BTFSC	TMR3H,7			;** Is Timer 3 MSbit set?
								;**   the "false" interrupt
		RETFIE					;** Return from this "false"
								;**   interrupt. Wait for
								;**   "real" overflow.
		MOVLB	01				;** Return to bank 1
		INCF	T3OFLCNTR,1		; increment the overflow counter
		GOTO	CK_CAP			; return to check if a RA1/CAP2
								;   interrupt also occured
;
; Other Interrupt routines. (Not utilized in this example)
;
EXT_INT	RETFIE					; RA0/INT interrupt routine
								;   (NOT used in this program)
RTCCINT	RETFIE					; RTCC overflow interrupt routine	
								;   (NOT used in this program)
RT_INT	RETFIE					; RA1/RT interrupt routine	
								;   (NOT used in this program)
								;
SRESET	GOTO	START			; If program became lost, goto
								;   START and reinitalize.
;
;
; When the executed address is NOT in the program range, the
; 16-bit address should contain all 1's (a CALL 0x1FFF). At
; this location you could branch to a routine to recover or
; shut down from the invalid program execution.
;
		ORG		END_OF_PROG_MEM	;  
		GOTO	SRESET			; The program has lost it's mind,
								;   do a system reset
		END
